/**************************************************************

File Name:	Matrix.asm
		
Date Modified:	05/22/00		GJO
				05/25/00		GJO	Code verified on rev 0.1 Silicon
				09/19/04		JDT Ported to VDSP++ 3.5. replace .SECTION syntax
Description:
			ADSP-21364 multichannel SIMD subroutine that implements the 
			multiplication of a matrix times a vector.  The input 
			matrices, a and b, and output matrix, c, are interleaved.
			
			Equation:	c[Mx1]=a[MxN]xb[Nx1]

Calling Parameters:
			b0=mat_a	l0=@mat_a		Matrix pointers and cicular buffers
			b1=mat_c	l1=0
			b8=mat_b	l8=@mat_b
			m0 = matrix dimension "M"
			m8 = matrix dimension "N"
			m1, m9 = address modifers

Assumptions:
			The three matrices have arbitrary dimensions. Matrix A is accessed as a 
			circular buffer so that the last iteration of the inner loop will do a 
			dummy read from a known location.

			BLOCK 1
			a[MxN] = { a[11],a'[11],a[12],a'[12],...,a[1N],a'[1N],a[21],a'[21],... }
			c[Mx1] = { c[11],c'[11],c[21],c'[21],...,c[M1],c'[M1] }

			BLOCK 0
			b[Nx1] = { b[11],b'[11],b[21],b'[21],...,b[N1],b'[N1] }

		
Registers Affected:
			r0,s0,r4,s4,r8,s8,r12,s12
			i0,i1,i8,

Cycle Count:
			cycles=5 + M(2+N) + 5 + 3	(entrance + core + 5 cache + exit)

			
Memory Usage:
			Instructions Words (48-bits):
				11 instruction words
			
			Data Words (16, 32, 40, or 64-bits):
				Number of data in Matrix a = 2(MxN) (32-bits)
				Number of data in Matrix b = 2(N) (32-bits)
				Number of data in Matrix a = 2(M) (32-bits)

Notes: 
		
**************************************************************/
#include "def21364.h"

.GLOBAL mxnxnx1;

.SECTION/PM     seg_pmco;
mxnxnx1:	bit set mode1 PEYEN;							/* Enable SIMD mode */
			nop;										
			r8=r8 xor r8, f0=dm(i0,m1), f4=pm(i8,m9); 		/* Clear f8 */
			f12=f0*f4,   f0=dm(i0,m1), f4=pm(i8,m9);
			lcntr=m0, do column until lce;					/* Column Loop */
		  		lcntr=m8, do row until lce;					/* Row Loop */
row:		    	f12=f0*f4, f8=f8+f12, f0=dm(i0,m1), f4=pm(i8,m9);
column:		  	r8=r8 xor r8, dm(i1,m1)=f8;
			rts (db);										/* Return to calling code */
			bit clr mode1 PEYEN;							/* Disable SIMD mode */
			nop;



